package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.*;

import java.time.Duration;
import java.util.ArrayList;
import java.util.List;

import static com.example.AkkaMainSystem.getDistributionCenter;

public class DeliveryCar extends AbstractBehavior<DeliveryCar.Message> {

    private final ArrayList<Paket> pakete = new ArrayList<>();
    private final ArrayList<ActorRef<Customer.Message>> route;
    private final int maxCapacity = 3;
    private final TimerScheduler<Message> timers;
    private int globalIndex = 0;

    public interface Message {}

    public record LoadMessage(List<Paket> pakete) implements Message {}

    public record PickupResponse(Paket paket) implements Message {}

    public record StartRoute() implements Message {}

    public record CheckStop() implements Message {}

    public record Clear() implements Message {} // Clear all packages in the delivery car

    private DeliveryCar(ActorContext<Message> context, TimerScheduler<Message> timers, ArrayList<ActorRef<Customer.Message>> route) {
        super(context);
        this.route = route;
        this.timers = timers;
    }

    public static Behavior<Message> create(List<ActorRef<Customer.Message>> route) {
        return Behaviors.setup(context -> Behaviors.withTimers(timers -> new DeliveryCar(context, timers, new ArrayList<>(route))));
    }

    @Override
    public Receive<Message> createReceive() {
        return newReceiveBuilder()
                .onMessage(PickupResponse.class, this::onPickupResponse)
                .onMessage(LoadMessage.class, this::onLoadMessage)
                .onMessage(StartRoute.class, this::onStartRoute)
                .onMessage(CheckStop.class, this::onCheckStop)
                .onMessage(Clear.class, this::onClear)
                .build();
    }

    private Behavior<Message> onLoadMessage(LoadMessage msg) {
        pakete.addAll(msg.pakete);
        getContext().getLog().info("Laden: {} Pakete wurden in den LKW geladen.", msg.pakete.size());
        scheduleNextStopCheck(Duration.ofSeconds(3)); // Start checking the first stop after 3 seconds
        return this;
    }

    private Behavior<Message> onPickupResponse(PickupResponse rsp) {
        if (rsp.paket != null && rsp.paket.inhalt != null && pakete.size() < maxCapacity) {
            pakete.add(rsp.paket);
            getContext().getLog().info("Abholung: Ein Paket wurde aufgenommen. Aktuelle Anzahl der Pakete im LKW: {}", pakete.size());
        }
        scheduleNextStopCheck(Duration.ofSeconds(1)); // Move to the next stop after 1 second
        return this;
    }

    private Behavior<Message> onCheckStop(CheckStop stop) {
        if (globalIndex >= route.size()) {
            getContext().getLog().info("Rückkehr: Der LKW kehrt zum Verteilzentrum zurück mit {} Paketen.", pakete.size());
            DistributionCenter.ArriveMessage antwort = new DistributionCenter.ArriveMessage(pakete, getContext().getSelf());
            getDistributionCenter().tell(antwort);
            globalIndex = 0;
        } else {
            ActorRef<Customer.Message> currentStop = route.get(globalIndex);
            getContext().getLog().info("Aktuelle Haltestelle: {}", currentStop.path().name());
            deliverPackages(currentStop);
            if (pakete.size() < maxCapacity) {
                sendPickupMessage(currentStop);
            }
            globalIndex++;
            scheduleNextStopCheck(pakete.size() >= maxCapacity ? Duration.ofSeconds(1) : Duration.ofSeconds(3));
        }
        return this;
    }

    private Behavior<Message> onStartRoute(StartRoute msg) {
        globalIndex = 0;
        scheduleNextStopCheck(Duration.ofSeconds(3)); // Start checking the first stop after 3 seconds
        return this;
    }

    private Behavior<Message> onClear(Clear clear) {
        pakete.clear();
        return this;
    }

    private void sendPickupMessage(ActorRef<Customer.Message> customer) {
        customer.tell(new Customer.PickUpMessage(getContext().getSelf(), "Der LKW kann das Paket abholen."));
    }

    private void deliverPackages(ActorRef<Customer.Message> customer) {
        pakete.removeIf(paket -> {
            if (paket.empfaenger.equals(customer)) {
                customer.tell(new Customer.DeliveryMessage(paket));
                return true;
            }
            return false;
        });
    }

    private void scheduleNextStopCheck(Duration duration) {
        timers.startSingleTimer(new CheckStop(), duration);
    }
}
